datum

	//============================================================================================
	var/et_Event/et_event

	//============================================================================================
	proc
		//
		//	PRIVATE
		//
		//----------------------------------------------------------------------------------------
		et_GetPrevious(et_Event/Event)
			var/et_Event
				current								= src.et_event
				last
			while(current)
				if (current == Event)				return last
				last								= current
				current								= current.et_nextEvent
			return null

		//----------------------------------------------------------------------------------------
		et_GetTail()
			var/et_Event
				current						= src.et_event
				last
			while(current)
				last								= current
				current								= current.et_nextEvent
			return last

		//
		//	PUBLIC
		//

		//----------------------------------------------------------------------------------------
		//	Add Event to the datum.
		//	If Merge then attempt to merge with the first matching et_Event.
		//	Otherwise, Event is added to the end.
		//	Returns TRUE on success.
		et_Add(et_Event/Event,Merge=TRUE)
			if ( !istype(Event) )	return FALSE

			var/et_Event/E

			if (Merge)	for(E in src.et_GetAddMatches(Event))	return E.et_AddMerge(Event)

			E			= src.et_GetTail()
			if (E)		E.et_nextEvent	= Event.et_Copy()
			else		src.et_event	= Event.et_Copy()

			return TRUE

		//----------------------------------------------------------------------------------------
		//	Remove all events.
		//	Returns TRUE if there were any events to remove.
		et_ClearEvents()
			if (src.et_event)
				src.et_event	= null
				//	The garbage handler should take care of the rest.
				return TRUE
			return FALSE

		//----------------------------------------------------------------------------------------
		//	Add all of src's events to Clone.
		//	Returns TRUE if there were events and they were all copied.
		et_CopyEventsTo(datum/Clone)
			if ( !istype(Clone) || !src.et_event)	return FALSE
			var/et_Event/current					= src.et_event
			.										= TRUE
			while(current)
				.									= (Clone.et_Add(current,FALSE) && .)
				current								= current.et_nextEvent

		//----------------------------------------------------------------------------------------
		//	Returns all events that are merge candidates with Match.
		et_GetAddMatches(et_Event/Match)
			.										= list()
			var/et_Event/current					= src.et_event
			while(current)
				if ( current.et_MatchesAdd(Match) )	.	+= current
				current								= current.et_nextEvent

		//----------------------------------------------------------------------------------------
		//	Returns all events that would be performed when Match is performed.
		et_GetPerformMatches(et_Event/Match)
			.												= list()
			var/et_Event/current							= src.et_event
			while(current)
				if ( current.et_MatchesPerform(Match) )		.	+= current
				current										= current.et_nextEvent

		//----------------------------------------------------------------------------------------
		//	Returns all events that are removal candidates with Match.
		et_GetRemoveMatches(et_Event/Match)
			.											= list()
			var/et_Event/current						= src.et_event
			while(current)
				if ( current.et_MatchesRemove(Match) )	.	+= current
				current									= current.et_nextEvent

		//----------------------------------------------------------------------------------------
		//	Perform all events that match Match.
		//	Arguments is for you to pass information to the events.
		et_PerformEvents(et_Event/Match,Arguments)
			.	= FALSE
			for(var/et_Event/E in src.et_GetPerformMatches(Match))
				.	= E.et_Perform(src,Arguments) || .

		//----------------------------------------------------------------------------------------
		//	Remove the first et_Event that matches Event.
		et_Remove(et_Event/Event)
			if ( !istype(Event) )		return FALSE

			var/candidates				= src.et_GetRemoveMatches(Event)
			if ( !length(candidates) )	return FALSE

			var/et_Event
				E				= candidates[1]
				previous		= src.et_GetPrevious(E)
			if (previous)		previous.et_nextEvent	= E.et_nextEvent
			else				src.et_event			= E.et_nextEvent
			E.et_nextEvent		= null

			return TRUE